// line 260 "ExpressionGeneralProposal.jweb"
/*;
Copyright (c) 1998 The Omega Project for Statistical Computing.  
All rights reserved.

*/


// line 242 "ExpressionGeneralProposal.jweb"
   package org.omegahat.Simulation.MCMC.Expressions;


// line 248 "ExpressionGeneralProposal.jweb"
    import org.omegahat.Simulation.MCMC.*;
    import org.omegahat.Simulation.MCMC.Listeners.*;
    import org.omegahat.Simulation.MCMC.Proposals.*;
    import org.omegahat.Simulation.MCMC.Targets.*;

import org.omegahat.Models.Formulae.Expressions.*;
import org.omegahat.Environment.Interpreter.*;
import org.omegahat.Environment.Parser.Parse.*;
import org.omegahat.Environment.Databases.*;
import java.util.Enumeration;

// line 34 "ExpressionGeneralProposal.jweb"
public class ExpressionGeneralProposal extends ExpressionMarkovProposal implements GeneralProposal
{


// line 47 "ExpressionGeneralProposal.jweb"
protected MethodCall[]  calls;
protected List[]        RHSarguments;
protected List[]        LHSarguments;



// line 58 "ExpressionGeneralProposal.jweb"
public ExpressionGeneralProposal( MultiLineModelFormula model, 
                                  Database environment ) throws Exception
{
  super( model, environment );
  doParsing();
}

public ExpressionGeneralProposal( String   model,
                                  Database environment        ) throws Exception
{
  super( model, environment );
  doParsing();
}

public ExpressionGeneralProposal( String[] model,
                                  Database environment ) throws Exception
{
  super( model, environment );
  doParsing();
}


// line 87 "ExpressionGeneralProposal.jweb"
protected void doParsing()
{
  RHSarguments = new List[ model.size() ];
  LHSarguments = new List[ model.size() ];
  calls = new MethodCall[ model.size() ];

  // Iterate over all the model expressions
  for(int index=0; index < model.size(); index++)
  {

    // get a line from the model
    ModelFormula f = (ModelFormula) model.elementAt(index);
    if(debug) System.err.println("Trying to handle line : " + f + " ." );

    // get the right hamd side of the expression
    FunctionTransformationExpression ft = (FunctionTransformationExpression) ((List) f.structural()).elementAt(0);

    
    if(debug) System.err.println("1");

    // now create a new MethodCall using the name of the object and adding the method "logPDF"
    calls[index] = new CachingMethodCall( ft.qualifier(), "logPDF", new List() );

    if(debug) System.err.println("2");

    RHSarguments[index] = ft.args();
    LHSarguments[index] = (List) f.response();

    if(debug) System.err.println("3");

  }
} 


// line 124 "ExpressionGeneralProposal.jweb"
protected SearchPath createPath( MCMCStateDatabase db )
  {
    SearchPath newPath = new SearchPath();
    // Scratch at 0
    newPath.attach( db, 1 );
    newPath.attach( environment, 2 );
    return newPath;
  }

public double unnormalizedConditionalPDF( Object to , Object from )
{
  return transitionProbability( from, to );
}

public double logUnnormalizedConditionalPDF( Object to, Object from )
{
  return logTransitionProbability( from, to );
}

public double conditionalPDF( Object to , Object from )
{
  return transitionProbability( from, to );
}

public double logConditionalPDF( Object to, Object from )
{
  return logTransitionProbability( from, to );
}

public double transitionProbability( Object from, Object to )
{
  return Math.exp( logTransitionProbability( from, to ) );
}

public double logTransitionProbability( Object fromState, Object toState )
  {
    // coerce inputs to something we can work with
    MCMCStateDatabase from = (MCMCStateDatabase) fromState;
    MCMCStateDatabase to   = (MCMCStateDatabase) toState;
    
    // create searchpaths 
    SearchPath fromPath = createPath( from );
    SearchPath toPath   = createPath( to   );
    
    double retval = 0.0;
    
    // evaluate each model term
    for( int i = 0; i < calls.length; i++)
    {
      // Declare outside of try blocks so that they endure
      Object fromVal;
      Object toVal;

      // first evaluate from arguments
      if(debug) System.err.println(" Trying to evaluate RHS arguments " +  RHSarguments[i] + " . ");
      searchPath(fromPath);
      try { fromVal = RHSarguments[i].eval(this); }
      catch( Throwable ex )
	{
	  ex.printStackTrace();
	  throw new RuntimeException("Failed to evaluate arguments " + RHSarguments[i] + " to call " + calls[i] + ".");
	}
      
      // now to arguments 
      if(debug) System.err.println(" Trying to evaluate LHS arguments " +  LHSarguments[i] + " . ");
      searchPath(toPath);
      try { toVal = LHSarguments[i].eval(this);}
      catch( Throwable ex )
	{
	  ex.printStackTrace();
	  throw new RuntimeException("Failed to evaluate arguments " + LHSarguments[i] + " to call " + calls[i] + ".");
	}
      
      // now the method call
      List allVals = new List();

      if(debug) System.err.println("Allvals = " + allVals );
      if( toVal instanceof List )  ((List) toVal).copyElements( allVals );
      else allVals.addElement( toVal );

      if(debug) System.err.println("Allvals = " + allVals );
      if( fromVal instanceof List )  ((List) fromVal).copyElements( allVals );
      else allVals.addElement( fromVal );

      if(debug) System.err.println("Allvals = " + allVals );
      calls[i].args(allVals);
      
      if (debug) System.err.println(" Evaluating method call "+ calls[i] + ".");   
      try
      {
	double tmp = ((Double) calls[i].eval(this )).doubleValue();
	retval += tmp;  // calls are to logPDF, so we need to accumulate via sum
	if(debug) System.err.println(" value = " + tmp + " retval = " + retval );
      }
      catch( Throwable ex )
	{
	  ex.printStackTrace();
	  throw new RuntimeException("Failed to evaluate transition probability for call " + calls[i] + ".");
	}
      

    }
    
    return retval;
    
  }
  
// line 42 "ExpressionGeneralProposal.jweb"
}
